# -*- coding: utf-8 -*-
"""Ammo factory test cases."""
import filecmp

import pytest

from yapam.armory import Armory
from yapam.config import ConfigRequest


pytestmark = [pytest.mark.armory]


@pytest.fixture
def config_requests(config_request):
    """List of config requests."""
    request_instance = ConfigRequest(**config_request)
    return [request_instance]


@pytest.fixture
def armory_instance(logger, temporary_ammo_file, config_requests):
    """Fixture with Armory instance."""
    return Armory(requests=config_requests, ammo_file_path=temporary_ammo_file, logger=logger)


@pytest.fixture
def phantom_ammo_blueprint(tmpdir_factory):
    """Fixture that create expected ammo file for compare."""
    fn = tmpdir_factory.mktemp('data').join('ammo')
    data = (
        '192 /auth\n'
        'POST /auth HTTP/1.1\r\n'
        'Host: 127.0.0.1:8888\r\n'
        'User-Agent: phantom\r\n'
        'Accept: */*\r\n'
        'Content-Type: application/json\r\n'
        'Connection: Close\r\n'
        'Content-Length: 42\r\n\r\n'
        '{"username": "admin", "password": "admin"}\r\n\r\n'
    )
    with open(fn, 'w', encoding='utf-8') as f:
        f.write(data)
    return fn


class TestArmory:
    """Armory test cases."""

    def test_init(self, logger, temporary_ammo_file, config_requests):
        """Armory constructor test case."""
        assert isinstance(Armory(requests=config_requests, ammo_file_path=temporary_ammo_file, logger=logger), Armory)

    def test_generate_ammo(self, armory_instance, phantom_ammo_blueprint):
        """Check that armory can generate phantom ammo as expected."""
        assert armory_instance.generate_ammo()
        assert filecmp.cmp(armory_instance.ammo_file_path, phantom_ammo_blueprint)



# -*- coding: utf-8 -*-

"""Yet another ammo generator for Yandex Tank.

for additional info see README.md
"""

__version__ = '0.1.0'
__all__ = [
    'armory', 'config', 'phantom'
]

__author__ = 'Aleksey Devyatkin <devyatkin.av@ya.ru>'



# -*- coding: utf-8 -*-
"""Common fixtures."""
import logging

import pytest


@pytest.fixture(scope='session')
def temporary_log_file(tmpdir_factory):
    """Temporary log file."""
    return tmpdir_factory.mktemp('data').join('tmp.log')


@pytest.fixture(scope='session')
def temporary_json_file(tmpdir_factory):
    """Temporary JSON file."""
    return tmpdir_factory.mktemp('data').join('cfg.json')


@pytest.fixture
def logger(temporary_log_file):
    """Fixture with logger instance."""
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.FileHandler(temporary_log_file))
    return logger


@pytest.fixture(scope='session')
def temporary_ammo_file(tmpdir_factory):
    """Temporary ammo file."""
    return str(tmpdir_factory.mktemp('data').join('ammo'))


@pytest.fixture
def config_request():
    """Fixture with parameters in the ConfigRequest."""
    request_dict = {
        'host': '127.0.0.1',
        'port': 8888,
        'url': '/auth',
        'method': 'POST',
        'body': '{\"username\": \"admin\", \"password\": \"admin\"}'
    }
    return request_dict



# -*- coding: utf-8 -*-
"""Config test cases."""
import json

import pytest

from yapam.config import AmmoConfig, ConfigRequest


pytestmark = [pytest.mark.config]


@pytest.fixture
def bad_config_request():
    """Fixture with parameters missing in the ConfigRequest."""
    request_dict = {
        'host': '127.0.0.1',
        'port': 8888,
        'url': '/auth',
        'method': 'POST',
        'body': '{\"username\": \"admin\", \"password\": \"admin\"}',
        'headers': 'headers'
    }
    return request_dict


@pytest.fixture(scope='session')
def good_config_file(tmpdir_factory):
    """Fixture with temporary config file."""
    fn = tmpdir_factory.mktemp('data').join('data.json')
    with open(fn, 'w', encoding='utf-8') as f:
        json.dump(AmmoConfig().template_blueprint, f, ensure_ascii=False, indent=4)
    return fn


@pytest.fixture(scope='session')
def bad_config_file(tmpdir_factory):
    """Fixture with temporary bad config file."""
    fn = tmpdir_factory.mktemp('data').join('data.json')
    data = AmmoConfig().template_blueprint
    # add bad parameter
    data['REQUESTS'] = [
        {
            'host': '127.0.0.1',
            'port': '8888',
            'url': '/auth',
            'method': 'BAD',
        }]
    with open(fn, 'w', encoding='utf-8') as f:
        json.dump(data, f, ensure_ascii=False, indent=4)
    return fn


@pytest.mark.request
class TestConfigRequest:
    """ConfigRequest test cases."""

    def test_good_request(self, config_request):
        """Check that expected request structure allowed."""
        request_instance = ConfigRequest(**config_request)
        assert isinstance(request_instance, ConfigRequest)

    def test_bad_request(self, bad_config_request):
        """Check that unexpected request structure not allowed."""
        try:
            ConfigRequest(**bad_config_request)
        except TypeError:
            assert True
        else:
            raise AssertionError()


class TestAmmoConfig:
    """AmmoConfig test cases."""

    def test_good_config_load(self, good_config_file):
        """Check that allowed structure of config file can be parsed."""
        cfg = AmmoConfig(good_config_file)
        assert cfg.requests

    def test_bad_config_load(self, bad_config_file):
        """Check that not allowed structure of config file can not be parsed."""
        try:
            AmmoConfig(bad_config_file)
        except TypeError as err:
            assert 'bad parameters' in str(err)
        else:
            raise AssertionError()

    def test_create_template(self, temporary_json_file):
        """Check that template with config blueprint can be created."""
        AmmoConfig().create_template(temporary_json_file)
        assert True



# -*- coding: utf-8 -*-
"""Ammo for phantom load generator."""


class PhantomAmmo:
    """PhantomAmmo blueprint.

    Result should look like:
        234 CONFIG_CASE1
        POST AUTH HTTP/1.1
        Authentication: jwt QWEASDASD
        Host: 127.0.0.1:80
        User-Agent: phantom
        Accept: */*
        Content-Type: application/json
        Connection: Close
        Content-Length: 54

        {"username": "tank_user_0", "password": "tank_user_0"}
    """

    no_body_template = (
        '{method} {url} HTTP/1.1\r\n'
        '{headers}'
    )

    body_template = (
        '{method} {url} HTTP/1.1\r\n'
        '{headers}\r\n'
        'Content-Length: {content_length}\r\n'
        '\r\n'
        '{body}'
    )
    default_headers = {'Host': None,
                       'User-Agent': 'phantom',
                       'Accept': '*/*',
                       'Content-Type': 'application/json',
                       'Connection': 'Close'}

    ammo_template = (
        '{request_length} {case}\n'
        '{request}\r\n\r\n'
    )

    def __init__(self, method: str, url: str, host: str, case: str, port: int,
                 extra_headers: dict,
                 body: str,
                 log=None):
        """Phantom-type bullet constructor.

        method: one of allowed http methods.
        url: url where load generator will shoot.
        host: host to shoot.
        case: test case tag in report.
        port: port where handler runs.
        extra_headers: request additional headers.
            default_headers are:
                {'Host': None,
                 'User-Agent': 'phantom',
                 'Accept': '*/*',
                 'Content-Type': 'application/json',
                 'Connection': 'Close'}
        body: request body.
        log: logger instance for debug messages.
        """
        self.log = log
        self.method = method
        self.url = url
        self.host = host

        self.port = port
        self.case = case

        headers = extra_headers
        self.default_headers['Host'] = '{host}:{port}'.format(host=self.host, port=self.port)
        headers.update(self.default_headers)

        self.headers = headers
        self.body = body

    @property
    def headers(self) -> str:
        """Phantom-type bullet http headers."""
        return self.__headers

    @headers.setter
    def headers(self, value: dict):
        """Phantom-type bullet http headers setter."""
        headers_list = list()
        for key, value in value.items():
            # all key words must be capitalized
            header_key = '-'.join([word.lower().capitalize() for word in key.split('-')])
            header_str = f'{header_key}: {value}'
            headers_list.append(header_str)
        self.__headers = '\r\n'.join(headers_list)

    @property
    def request(self):
        """Phantom-type bullet http request."""
        if self.body:
            return self.body_template.format(method=self.method, url=self.url, headers=self.headers,
                                             content_length=len(self.body), body=self.body)
        return self.no_body_template.format(method=self.method, url=self.url, headers=self.headers)

    @property
    def bullet(self):
        """Phantom-type bullet for load generator."""
        request = self.request
        ammo = self.ammo_template.format(request_length=len(request), case=self.case, request=request)
        if self.log:
            self.log.debug(ammo.replace('\r\n', ', ').replace('\n', ', '))
        return ammo



# -*- coding: utf-8 -*-
"""Ammo factory.

Supported ammo types: Phantom
In most cases, you need this particular module.

Example of usage:
    user_config = AmmoConfig('config.json')
    armory = Armory(user_config.requests, user_config.ammo_file, user_config.log)
    armory.generate_ammo()
"""

from dav_utils.config import Config
from dav_utils.descriptors import WritableFile
from dav_utils.utils import Util

from yapam.phantom import PhantomAmmo


class Armory(Util):
    """Ammo factory.

    requests:       list of ConfigRequest instances, like [ConfigRequest, ]
    ammo_file_path: path to file where result should be saved.
    logger:         your logger instance. Transmitted to ammo class for debug logging. May be None.
    """

    ammo_file_path = WritableFile('ammo_file_path')

    def __init__(self, requests: str, ammo_file_path: str, logger: Config.log):
        """Armory constructor.

        requests:  list of requests from config
        ammo_file: path to a file where results should be saved
        logger:    config logger.
        """
        self.requests = requests
        self.ammo_file_path = ammo_file_path
        self.log = logger

    def generate_ammo(self):
        """Generate and write Phantom ammo to a text file."""
        ammo_gen = (
            PhantomAmmo(log=self.log,
                        method=request.method,
                        url=request.url,
                        host=request.host,
                        case=request.case,
                        port=request.port,
                        extra_headers=request.extra_headers,
                        body=request.body).bullet for request in self.requests
        )
        self.save_text_file(file_path=self.ammo_file_path, txt_data=ammo_gen)
        return True



# -*- coding: utf-8 -*-
"""Phantom test cases."""
import pytest

from yapam.phantom import PhantomAmmo


pytestmark = [pytest.mark.phantom]


@pytest.fixture
def phantom_ammo_dict(logger):
    """Fixture with PhantomAmmo parameters without body."""
    ammo_dict = {
        'host': '127.0.0.1',
        'port': 8888,
        'url': '/auth',
        'method': 'GET',
        'log': logger,
        'case': 'tests',
        'extra_headers': {'Authorization': 'token'},
        'body': ''
    }
    return ammo_dict


@pytest.fixture
def phantom_ammo_with_body_dict(phantom_ammo_dict):
    """Fixture with PhantomAmmo parameters with body."""
    ammo_dict = phantom_ammo_dict
    ammo_dict['method'] = 'POST'
    ammo_dict['body'] = '{\"username\": \"admin\", \"password\": \"admin\"}'
    ammo_dict['log'] = None
    return ammo_dict


@pytest.fixture
def phantom_headers_str():
    """Fixture with expected default headers converted to str."""
    headers_str = ('Authorization: token\r\n'
                   'Host: 127.0.0.1:8888\r\n'
                   'User-Agent: phantom\r\n'
                   'Accept: */*\r\n'
                   'Content-Type: application/json\r\n'
                   'Connection: Close'
                   )
    return headers_str


@pytest.fixture
def phantom_headers_with_body_str(phantom_headers_str):
    """Fixture with expected default headers for request with body converted to str."""
    return '\r\n'.join([phantom_headers_str, 'Content-Length: 42'])


@pytest.fixture
def phantom_request_str(phantom_headers_str):
    """Fixture with expected Phantom request str."""
    request_str = 'GET /auth HTTP/1.1\r\n' + phantom_headers_str
    return request_str


@pytest.fixture
def phantom_request_with_body_str(phantom_headers_with_body_str):
    """Fixture with expected Phantom request with body str."""
    return '\r\n'.join(
        ['POST /auth HTTP/1.1', phantom_headers_with_body_str, '\r\n{"username": "admin", "password": "admin"}'])


@pytest.fixture
def phantom_bullet_str(phantom_request_str):
    """Fixture with expected phantom bullet."""
    return '147 tests\n' + phantom_request_str + '\r\n\r\n'


@pytest.fixture
def phantom_bullet_with_body_str(phantom_request_with_body_str):
    """Fixture with expected phantom bullet for request with body."""
    return '214 tests\n' + phantom_request_with_body_str + '\r\n\r\n'


@pytest.fixture
def phantom_ammo_inst(phantom_ammo_dict):
    """Fixture with PhantomAmmo (without body) instance."""
    return PhantomAmmo(**phantom_ammo_dict)


@pytest.fixture
def phantom_ammo_with_body_inst(phantom_ammo_with_body_dict):
    """Fixture with PhantomAmmo (with body) instance."""
    return PhantomAmmo(**phantom_ammo_with_body_dict)


class TestPhantomAmmo:
    """Phantom ammo without body test cases."""

    def test_init(self, phantom_ammo_dict):
        """Constructor test."""
        assert isinstance(PhantomAmmo(**phantom_ammo_dict), PhantomAmmo)

    def test_headers_property(self, phantom_ammo_inst, phantom_headers_str):
        """Check that headers property converted as expected."""
        assert phantom_ammo_inst.headers == phantom_headers_str

    def test_request_property(self, phantom_ammo_inst, phantom_request_str):
        """Check that request property converted as expected."""
        assert phantom_ammo_inst.request == phantom_request_str

    def test_bullet_property(self, phantom_ammo_inst, phantom_bullet_str, temporary_log_file):
        """Check that bullet property converted as expected."""
        assert phantom_ammo_inst.bullet == phantom_bullet_str

        def last_log_line(file_name: str):
            """Return last string from temporary log."""
            with open(file_name, 'r') as log_file:
                lines = log_file.readlines()
            return lines[-1]
        for attr in phantom_bullet_str.split('\n'):
            assert attr.replace('\r', '') in last_log_line(temporary_log_file)


class TestPhantomAmmoWithBody:
    """Phantom ammo with body test cases."""

    def test_init(self, phantom_ammo_with_body_dict):
        """Constructor test."""
        assert isinstance(PhantomAmmo(**phantom_ammo_with_body_dict), PhantomAmmo)

    def test_headers_property(self, phantom_ammo_with_body_inst, phantom_headers_str):
        """Check that headers property converted as expected."""
        assert phantom_ammo_with_body_inst.headers == phantom_headers_str

    def test_request_property(self, phantom_ammo_with_body_inst, phantom_request_with_body_str):
        """Check that request property converted as expected."""
        assert phantom_ammo_with_body_inst.request == phantom_request_with_body_str

    def test_bullet_property(self, phantom_ammo_with_body_inst, phantom_bullet_with_body_str):
        """Check that bullet property converted as expected."""
        assert phantom_ammo_with_body_inst.bullet == phantom_bullet_with_body_str



# -*- coding: utf-8 -*-
"""Project config."""

from dav_utils.config import Config
from dav_utils.descriptors import (DictType, HttpMethod, IntType,
                                   StringType)


class ConfigRequest:
    """Structure of Config.requests list element.

    host:           request host parameter (host where load generator will shoot).
    url:            request url parameter (handler where load generator will shoot).
    method:         request method.
    case:           test case tag in report. default value is url.
    extra_headers:  additional request headers (dict)
    body:           request body string.
    port:           request port where handler runs. default value is 80.
    """

    method = HttpMethod('method')
    url = StringType('url')
    host = StringType('host')
    body = StringType('body')
    port = IntType('port')
    extra_headers = DictType('extra_headers')

    def __init__(self, host: str, url: str, method: str, case: str = None, port: int = 80, extra_headers: dict = None,
                 body: str = None):
        """Validate parameters and create instance of ConfigRequest."""
        self.method = method
        self.url = url
        self.host = host
        self.port = port
        self.case = case if case else url
        self.extra_headers = extra_headers if extra_headers else dict()
        self.body = body if body else ''


class ConfigRequestType:
    """Descriptor for ConfigRequestType checking."""

    def __init__(self, name):
        """Set attribute name."""
        self.name = name

    def __set__(self, instance, raw_values_list: list):
        """Check that raw_value_list is a list and create ConfigRequest instance for each list element."""
        if isinstance(raw_values_list, list):
            try:
                instance.__dict__[self.name] = [ConfigRequest(**value) for value in raw_values_list]
            except TypeError:
                raise TypeError(f'{self.name} contains bad parameters.')
        else:
            raise TypeError(f'{self.name} should be a list.')

    def __get__(self, instance, class_):
        """Return attribute value."""
        return instance.__dict__[self.name]


class AmmoConfig(Config):
    """Script configuration.

    Factory parameters:
        ammo_file: path to a file where results should be saved
        requests: list of a request-hashes from config
            "REQUESTS": [
                {
                  "host": "127.0.0.1",      # request host parameter (where load generator will shoot).
                  "url":  "AUTH",           # server handler URL (where load generator will shoot).
                  "method":  "POST",        # request method.
                  "case":  "CONFIG_CASE1",  # test case tag in report.
                  "extra_headers": {}       # additional request headers.
                  "body": "user=user"       # body string (json-type)
                  "port": 443               # port where handler runs. default value is 80. int.
                }
            ]

    script logging:
        log_date_fmt: log date format (only str)
        log_fmt: log format (only str)
        log_lvl: log level (logging.DEBUG, logging.INFO and etc.)
    """

    requests = ConfigRequestType('requests')
    ammo_file = StringType('ammo_file')

    @property
    def template_blueprint(self):
        """Template blueprint."""
        config_template = {
            'LOG_DATE_FMT': '%H:%M:%S',
            'LOG_FMT': '%(asctime)s.%(msecs)d|%(levelname).1s|%(message)s',
            'LOG_LVL': 'DEBUG',
            'AMMO_FILE': 'ammo',
            'REQUESTS': [
                {
                    'host': '127.0.0.1',
                    'port': 80,
                    'url': 'AUTH',
                    'method': 'POST',
                    'body': '{\"username\": \"tank_user_0\", \"password\": \"tank_user_0\"}'
                }]
        }
        return config_template

    def create_template(self, file_path: str):
        """Create JSON config file template."""
        self.save_json_file(file_path, self.template_blueprint)
        self.log.info('Template {file_path} created.'.format(file_path=file_path))



# -*- coding: utf-8 -*-
"""Python package config."""
import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='yapam',
    version='0.1.0',
    author='Aleksey Devyatkin',
    author_email='devyatkin.av@ya.ru',
    description='Yet another ammo generator for Yandex Tank',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/devalv/yapam',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=['dav-utils==0.2.*']
)
